@@ -1,6 +1,6 @@
 package Bitcoin::Crypto::Config;
 
-use Modern::Perl "2010";
+use v5.10; use warnings;
 use Exporter qw(import);
 
 our @EXPORT = qw(
