@@ -1,19 +1,22 @@
 package Bitcoin::Crypto::Key::ExtPrivate;
 
-use Modern::Perl "2010";
+use v5.10; use warnings;
 use Moo;
 use Crypt::Mac::HMAC qw(hmac);
-use Math::BigInt 1.999816 try => 'GMP';
-use Math::EllipticCurve::Prime;
 use Encode qw(encode decode);
 use Unicode::Normalize;
-use Bitcoin::BIP39 qw(gen_bip39_mnemonic bip39_mnemonic_to_entropy);
+use Bitcoin::BIP39 qw(gen_bip39_mnemonic bip39_mnemonic_to_entropy entropy_to_bip39_mnemonic);
 use Crypt::KeyDerivation qw(pbkdf2);
+use Scalar::Util qw(blessed);
 
 use Bitcoin::Crypto::Key::ExtPublic;
 use Bitcoin::Crypto::Config;
-use Bitcoin::Crypto::Helpers qw(pad_hex ensure_length);
+use Bitcoin::Crypto::Helpers qw(new_bigint pad_hex ensure_length verify_bytestring);
 use Bitcoin::Crypto::Exception;
+use Bitcoin::Crypto;
+
+use namespace::clean;
+our $VERSION = Bitcoin::Crypto->VERSION;
 
 with "Bitcoin::Crypto::Role::ExtendedKey";
 
@@ -31,8 +34,27 @@ sub generate_mnemonic
 		"required entropy of between $min_len and $max_len bits, divisible by $len_div"
 	) if $len < $min_len || $len > $max_len || $len % $len_div != 0;
 
-	my $ret = gen_bip39_mnemonic(bits => $len, language => $lang);
-	return $ret->{mnemonic};
+	return Bitcoin::Crypto::Exception::MnemonicGenerate->trap_into(
+		sub {
+			my $ret = gen_bip39_mnemonic(bits => $len, language => $lang);
+			$ret->{mnemonic};
+		}
+	);
+}
+
+sub mnemonic_from_entropy
+{
+	my ($class, $entropy, $lang) = @_;
+	$lang //= "en";
+
+	return Bitcoin::Crypto::Exception::MnemonicGenerate->trap_into(
+		sub {
+			entropy_to_bip39_mnemonic(
+				entropy => $entropy,
+				language => $lang
+			);
+		}
+	);
 }
 
 sub from_mnemonic
@@ -42,9 +64,17 @@ sub from_mnemonic
 	$password = encode("UTF-8", NFKD(decode("UTF-8", "mnemonic" . ($password // ""))));
 
 	if (defined $lang) {
+
 		# checks validity of seed in given language
 		# requires Wordlist::LANG::BIP39 module for given LANG
-		bip39_mnemonic_to_entropy(mnemonic => $mnemonic, language => $lang);
+		Bitcoin::Crypto::Exception::MnemonicCheck->trap_into(
+			sub {
+				bip39_mnemonic_to_entropy(
+					mnemonic => $mnemonic,
+					language => $lang
+				);
+			}
+		);
 	}
 	my $bytes = pbkdf2($mnemonic, $password, 2048, "SHA512", 64);
 
@@ -54,6 +84,8 @@ sub from_mnemonic
 sub from_seed
 {
 	my ($class, $seed) = @_;
+	verify_bytestring($seed);
+
 	my $bytes = hmac("SHA512", "Bitcoin seed", $seed);
 	my $key = substr $bytes, 0, 32;
 	my $cc = substr $bytes, 32, 32;
@@ -90,23 +122,27 @@ sub _derive_key_partial
 
 	my $hmac_data;
 	if ($hardened) {
+
 		# zero byte
 		$hmac_data .= "\x00";
+
 		# key data - 32 bytes
 		$hmac_data .= ensure_length $self->raw_key, $config{key_max_length};
-	} else {
+	}
+	else {
 		# public key data - SEC compressed form
 		$hmac_data .= $self->raw_key("public_compressed");
 	}
+
 	# child number - 4 bytes
 	$hmac_data .= ensure_length pack("N", $child_num), 4;
 
 	my $data = hmac("SHA512", $self->chain_code, $hmac_data);
 	my $chain_code = substr $data, 32, 32;
 
-	my $number = Math::BigInt->from_bytes(substr $data, 0, 32);
-	my $key_num = Math::BigInt->from_bytes($self->raw_key);
-	my $n_order = Math::EllipticCurve::Prime->from_name($config{curve_name})->n;
+	my $number = new_bigint(substr $data, 0, 32);
+	my $key_num = new_bigint($self->raw_key);
+	my $n_order = new_bigint(pack "H*", $self->key_instance->curve2hash->{order});
 
 	Bitcoin::Crypto::Exception::KeyDerive->raise(
 		"key $child_num in sequence was found invalid"
@@ -115,7 +151,11 @@ sub _derive_key_partial
 	$number->badd($key_num);
 	$number->bmod($n_order);
 
-	return __PACKAGE__->new(
+	Bitcoin::Crypto::Exception::KeyDerive->raise(
+		"key $child_num in sequence was found invalid"
+	) if $number->beq(0);
+
+	return (blessed $self)->new(
 		$number->as_bytes,
 		$chain_code,
 		$child_num,
@@ -186,6 +226,20 @@ Other languages than english require additional modules for L<Bitcoin::BIP39>.
 Dies when $len is invalid (under 128, above 256 or not divisible by 32).
 Returns newly generated BIP39 mnemonic string.
 
+In some environments a problem may be encountered that causes the secure random bytes generator to block the program execution (See L<Bytes::Random::Secure/"BLOCKING ENTROPY SOURCE">). In this case you can use I<mnemonic_from_entropy> and pass in entropy generated by L<Bytes::Random::Secure> in non-blocking mode (via the OO interface).
+
+=head2 mnemonic_from_entropy
+
+	sig: mnemonic_from_entropy($class, $bytes, $lang = "en")
+
+Generates a new mnemonic code from custom entropy given in $bytes (a bytestring). This entropy should be of the same bit size as in L</"generate_mnemonic">. Returns newly generated BIP39 mnemonic string.
+
+This can be useful to avoid relying on the underlying implementation of L<Bitcoin::BIP39>.
+
+Another use would be implementing one's own entropy source that can be truly random, not just cryptographically-secure. A popular example would be capturing user's mouse movements.
+
+Be aware that the method you use to generate a mnemonic will be a very important factor in your key's security. If possible, use real sources of randomness (not pseudo-random) or a cryptographically secure pseduo-random number generator like the one used by L<Bytes::Random::Secure>.
+
 =head2 from_mnemonic
 
 	sig: from_mnemonic($class, $mnemonic, $password = "", $lang = undef)
@@ -262,11 +316,11 @@ Returns the key in basic format: L<Bitcoin::Crypto::Key::Private>
 
 	sig: derive_key($self, $path)
 
-Performs extended key deriviation as specified in BIP32 on the current key with $path. Dies on error.
+Performs extended key derivation as specified in BIP32 on the current key with $path. Dies on error.
 
-See BIP32 document for details on deriviation paths and methods.
+See BIP32 document for details on derivation paths and methods.
 
-Returns a new extended key instance - result of a deriviation.
+Returns a new extended key instance - result of a derivation.
 
 =head2 get_fingerprint
 
@@ -282,6 +336,8 @@ This module throws an instance of L<Bitcoin::Crypto::Exception> if it encounters
 
 =item * MnemonicGenerate - mnemonic couldn't be generated correctly
 
+=item * MnemonicCheck - mnemonic didn't pass the validity check
+
 =item * KeyDerive - key couldn't be derived correctly
 
 =item * KeyCreate - key couldn't be created correctly
